// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SceneDebugLine.hpp"
#include "display/shader/SimpleTexturedShader.hpp"
#include "scene/components/display/Camera.hpp"
#include "scene/Scene.hpp"

using namespace Dawn;

struct ShaderPassItem SceneDebugLine::createShaderItem(
  Mesh *mesh,
  int32_t *lineIndex,
  Camera *camera,
  SimpleTexturedShader *shader
) {
  struct ShaderPassItem item;
  item.priority = SCENE_DEBUG_LINE_PRIORITY;

  item.shaderProgram = &shader->program;
  item.colorValues[shader->program.paramColor] = this->color;
  item.matrixValues[shader->program.paramModel] = this->translation;
  item.matrixValues[shader->program.paramView] = camera->transform->getWorldTransform();
  item.matrixValues[shader->program.paramProjection] = camera->projection;
  item.boolValues[shader->program.paramHasTexture] = false;

  item.renderFlags = 0x00;

  auto i = *lineIndex;
  item.mesh = mesh;
  item.drawMode = MESH_DRAW_MODE_LINES;
  item.start = i * SCENE_DEBUG_LINE_INDICE_COUNT;
  item.count = SCENE_DEBUG_LINE_INDICE_COUNT;

  glm::vec3 positions[SCENE_DEBUG_LINE_VERTICE_COUNT] = { this->v0, this->v1 };
  mesh->bufferPositions(
    i * SCENE_DEBUG_LINE_VERTICE_COUNT,
    positions,
    SCENE_DEBUG_LINE_VERTICE_COUNT
  );

  meshindice_t indices[SCENE_DEBUG_LINE_INDICE_COUNT] = {
    i * SCENE_DEBUG_LINE_VERTICE_COUNT,
    (i*SCENE_DEBUG_LINE_VERTICE_COUNT) + 1
  };
  mesh->bufferIndices(
    i * SCENE_DEBUG_LINE_INDICE_COUNT,
    indices,
    SCENE_DEBUG_LINE_INDICE_COUNT
  );
  
  (*lineIndex)++;
  return item;
}

// Scene Implementations (done here to keep things cleaner in scene)
void Scene::debugLine(struct SceneDebugLine line) {
  this->debugLines.push_back(line);
}

void Scene::debugCube(struct SceneDebugCube cube) {
  auto min = cube.min;
  auto max = cube.max;

  struct SceneDebugLine line;
  line.color = cube.color;
  line.translation = cube.translation;

  // Bottom Face
  line.v0 = glm::vec3(min.x, min.y, min.z), line.v1 = glm::vec3(max.x, min.y, min.z);
  this->debugLine(line);
  line.v0 = glm::vec3(min.x, min.y, min.z), line.v1 = glm::vec3(min.x, min.y, max.z);
  this->debugLine(line);
  line.v0 = glm::vec3(max.x, min.y, min.z), line.v1 = glm::vec3(max.x, min.y, max.z);
  this->debugLine(line);
  line.v0 = glm::vec3(min.x, min.y, max.z), line.v1 = glm::vec3(max.x, min.y, max.z);
  this->debugLine(line);

  // Top Face
  line.v0 = glm::vec3(min.x, max.y, min.z), line.v1 = glm::vec3(max.x, max.y, min.z);
  this->debugLine(line);
  line.v0 = glm::vec3(min.x, max.y, min.z), line.v1 = glm::vec3(min.x, max.y, max.z);
  this->debugLine(line);
  line.v0 = glm::vec3(max.x, max.y, min.z), line.v1 = glm::vec3(max.x, max.y, max.z);
  this->debugLine(line);
  line.v0 = glm::vec3(min.x, max.y, max.z), line.v1 = glm::vec3(max.x, max.y, max.z);
  this->debugLine(line);

  // Rails
  line.v0 = glm::vec3(min.x, min.y, min.z), line.v1 = glm::vec3(min.x, max.y, min.z);
  this->debugLine(line);
  line.v0 = glm::vec3(max.x, min.y, min.z), line.v1 = glm::vec3(max.x, max.y, min.z);
  this->debugLine(line);
  line.v0 = glm::vec3(min.x, min.y, max.z), line.v1 = glm::vec3(min.x, max.y, max.z);
  this->debugLine(line);
  line.v0 = glm::vec3(max.x, min.y, max.z), line.v1 = glm::vec3(max.x, max.y, max.z);
  this->debugLine(line);
}